# -*- coding: utf-8 -*-
#
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  LSE
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''
  Description:  This file contains the definition of the remote token class
  
  Dependencies: -

'''

import logging
import copy
import traceback


import httplib2
import urllib
import json

from privacyidea.lib.config import getFromConfig
from privacyidea.lib.util import getParam
from privacyidea.lib.validate import split_pin_otp
from privacyidea.lib.validate import check_pin
from privacyidea.lib.log import log_with

optional = True
required = False

from privacyidea.lib.tokenclass import TokenClass


log = logging.getLogger(__name__)

###############################################


class RemoteTokenClass(TokenClass):
    """
    The Remote token forwards an authentication request to another privacyIDEA
    server. The request can be forwarded to a user on the other server or to
    a serial number on the other server. The PIN can be checked on the local
    privacyIDEA server or on the remote server.

    Using the Remote token you can assign one physical token to many
    different users.
    """

    def __init__(self, aToken):
        """
        constructor - create a token class object with it's db token binding

        :param aToken: the db bound token
        """
        TokenClass.__init__(self, aToken)
        self.setType(u"remote")

        self.remoteServer = ""
        self.remoteLocalCheckpin = None
        self.remoteSerial = None
        self.remoteUser = None
        self.remoteRealm = None
        self.remoteResConf = None
        self.mode = ['authenticate', 'challenge']

    @classmethod
    def getClassType(cls):
        """
        return the class type identifier
        """
        return "remote"

    @classmethod
    def getClassPrefix(cls):
        """
        return the token type prefix
        """
        return "LSRE"

    @classmethod
    @log_with(log)
    def getClassInfo(cls, key=None, ret='all'):
        """
        getClassInfo - returns a subtree of the token definition

        :param key: subsection identifier
        :type key: string

        :param ret: default return value, if nothing is found
        :type ret: user defined

        :return: subsection if key exists or user defined
        :rtype: s.o.

        """
        res = {'type': 'remote',
               'title': 'Remote Token',
               'description': ('REMOTE token to forward the authentication'
                               ' request to another privacyIDEA server'),

               'init': {'page': {'html': 'remotetoken.mako',
                                 'scope': 'enroll', },
                        'title': {'html': 'remotetoken.mako',
                                  'scope': 'enroll.title', },
                        },

               'config': {'page': {'html': 'remotetoken.mako',
                                   'scope': 'config', },
                          'title': {'html': 'remotetoken.mako',
                                    'scope': 'config.title', },
                          },

               'selfservice': {},
               'policy': {},
               }

        if key is not None and key in res:
            ret = res.get(key)
        else:
            if ret == 'all':
                ret = res
        return ret

    def update(self, param):
        """
        second phase of the init process - updates parameters

        :param param: the request parameters
        :return: - nothing -
        """

        self.remoteServer = getParam(param, "remote.server", required)
        # if another OTP length would be specified in /admin/init this would
        # be overwritten by the parent class, which is ok.
        self.setOtpLen(6)

        val = getParam(param, "remote.local_checkpin", optional)
        if val is not None:
            self.remoteLocalCheckpin = val

        val = getParam(param, "remote.serial", optional)
        if val is not None:
            self.remoteSerial = val

        val = getParam(param, "remote.user", optional)
        if val is not None:
            self.remoteUser = val

        val = getParam(param, "remote.realm", optional)
        if val is not None:
            self.remoteRealm = val

        val = getParam(param, "remote.resConf", optional)
        if val is not None:
            self.remoteResConf = val

        TokenClass.update(self, param)

        self.addToTokenInfo("remote.server", self.remoteServer)
        self.addToTokenInfo("remote.serial", self.remoteSerial)
        self.addToTokenInfo("remote.user", self.remoteUser)
        self.addToTokenInfo("remote.local_checkpin", self.remoteLocalCheckpin)
        self.addToTokenInfo("remote.realm", self.remoteRealm)
        self.addToTokenInfo("remote.resConf", self.remoteResConf)

        return

    @log_with(log)
    def check_pin_local(self):
        """
        lookup if pin should be checked locally or on remote host

        :return: bool
        """
        local_check = False
        if 1 == int(self.getFromTokenInfo("remote.local_checkpin")):
            local_check = True
        log.debug(" local checking pin? %r" % local_check)

        return local_check


    @log_with(log)
    def authenticate(self, passw, user, options=None):
        """
        do the authentication on base of password / otp and user and
        options, the request parameters.

        Here we contact the other privacyIDEA server to validate the OtpVal.

        :param passw: the password / otp
        :param user: the requesting user
        :param options: the additional request parameters

        :return: tuple of (success, otp_count - 0 or -1, reply)

        """
        log.debug("authenticate")

        res = False
        otp_counter = -1
        reply = None

        otpval = passw

        ## should we check the pin localy??
        if self.check_pin_local():
            (res, pin, otpval) = split_pin_otp(self, passw, user,
                                               options=options)

            res = TokenClass.checkPin(self, pin)
            if res is False:
                return (res, otp_counter, reply)

        (res, otp_count, reply) = self.do_request(otpval, user=user)

        return (res, otp_count, reply)

    @log_with(log)
    def is_challenge_request(self, passw, user, options=None):
        """
        This method checks, if this is a request, that triggers a challenge.
        It depends on the way, the pin is checked - either locally or remote

        :param passw: password, which might be pin or pin+otp
        :type passw: string
        :param user: The user from the authentication request
        :type user: User object
        :param options: dictionary of additional request parameters
        :type options: dict

        :return: true or false
        """

        request_is_valid = False

        if self.check_pin_local():
            pin_match = check_pin(self, passw, user=user, options=options)
            if pin_match is True:
                request_is_valid = True

        return request_is_valid

    @log_with(log)
    def do_request(self, passw, transactionid=None, user=None):
        """
        run the http request against the remote host

        :param passw: the password which should be checked on the remote host
        :param transactionid: provided,  if this is a challenge response
        :param user: the requesting user - used if no remote serial or remote
                     user is provided

        :return: Tuple of (success, otp_count= -1 or 0, reply=remote response)
        """

        reply = {}
        otpval = passw.encode("utf-8")

        remoteServer = self.getFromTokenInfo("remote.server") or ""
        remoteServer = remoteServer.encode("utf-8")

        ## in preparation of the ability to relocate privacyidea urls,
        ## we introduce the remote url path
        remotePath = self.getFromTokenInfo("remote.path") or ""
        remotePath = remotePath.strip().encode('utf-8')

        remoteSerial = self.getFromTokenInfo("remote.serial") or ""
        remoteSerial = remoteSerial.encode('utf-8')

        remoteUser = self.getFromTokenInfo("remote.user") or ""
        remoteUser = remoteUser.encode('utf-8')

        remoteRealm = self.getFromTokenInfo("remote.realm") or ""
        remoteRealm = remoteRealm.encode('utf-8')

        remoteResConf = self.getFromTokenInfo("remote.resConf") or ""
        remoteResConf = remoteResConf.encode('utf-8')

        ssl_verify = getFromConfig("remote.verify_ssl_certificate",
                                   False) or False

        if type(ssl_verify) in [str, unicode]:
            if ssl_verify.lower() == "true":
                ssl_verify = True
            else:
                ssl_verify = False


        ## here we also need to check for remote.user and so on....
        log.debug("checking OTP len:%r remotely on server: %r,"
                  " serial: %r, user: %r" %
                  (len(otpval), remoteServer, remoteSerial, remoteUser))
        params = {}

        if len(remoteSerial) > 0:
            params['serial'] = remoteSerial
            if len(remotePath) == 0:
                remotePath = "/validate/check_s"
        elif len(remoteUser) > 0:
            params['user'] = remoteUser
            params['realm'] = remoteRealm
            params['resConf'] = remoteResConf
            if len(remotePath) == 0:
                remotePath = "/validate/check"

        else:
        ## There is no remote.serial and no remote.user, so we will
        ## try to pass the requesting user.
            if user is None:
                log.warning("FIXME: We do not know the user at the moment!")
            else:
                params['user'] = user.login
                params['realm'] = user.realm

        params['pass'] = otpval
        if transactionid is not None:
            params['state'] = transactionid

        ## use a POST request to check the token
        data = urllib.urlencode(params)
        request_url = "%s%s" % (remoteServer, remotePath)

        try:
            ## prepare the submit and receive headers
            headers = {"Content-type": "application/x-www-form-urlencoded",
                       "Accept": "text/plain"}

            ## submit the request
            try:
                ## is httplib compiled with ssl?
                http = httplib2.Http(disable_ssl_certificate_validation=
                                     not(ssl_verify))
            except TypeError as exx:
                ## not so on squeeze:
                ## TypeError: __init__() got an unexpected keyword argument
                ## 'disable_ssl_certificate_validation'

                log.warning("httplib2 'disable_ssl_certificate_validation' "
                            "attribute error: %r" % exx)
                ## so we run in fallback mode
                http = httplib2.Http()

            (resp, content) = http.request(request_url,
                                           method="POST",
                                           body=data,
                                           headers=headers)
            result = json.loads(content)
            log.debug(result)
            status = result['result']['status']
            log.debug(status)

            if True == status:
                if True == result['result']['value']:
                    res = True
                    otp_count = 0

            if "detail" in result:
                reply = copy.deepcopy(result["detail"])
                otp_count = -1
                res = False

        except Exception as exx:
            log.error("Error getting response from "
                      "remote Server (%r): %r" % (request_url, exx))
            log.error(traceback.format_exc())

        return (res, otp_count, reply)

    @log_with(log)
    def checkResponse4Challenge(self, user, passw, options=None,
                                challenges=None):
        '''
        This method verifies if the given ``passw`` matches any
        existing ``challenge`` of the token.

        It then returns the new otp_counter of the token and the
        list of the matching challenges.

        In case of success the otp_counter needs to be >= 0.
        The matching_challenges is passed to the method
        :py:meth:`~privacyidea.lib.tokenclass.TokenClass.challenge_janitor`
        to clean up challenges.

        :param user: the requesting user
        :type user: User object
        :param passw: the password (pin+otp)
        :type passw: string
        :param options:  additional arguments from the request, which could
                         be token specific
        :type options: dict
        :param challenges: A sorted list of valid challenges for this token.
        :type challenges: list
        :return: tuple of (otpcounter and the list of matching challenges)

        '''
        otp_counter = -1
        transid = None
        matching_challenge = None
        reply = None

        matching_challenges = []

        if 'transactionid' in options or 'state' in options:
            ## fetch the transactionid
            transid = options.get('transactionid', options.get('state', None))

        if transid is not None:
            ## in case of a local pin check, we the transaction is a local one
            ## and we must not forward this!!
            if self.check_pin_local():
                ## check if transaction id is in list of challengens
                for challenge in challenges:
                    if challenge.transid == transid:
                        matching_challenge = challenge
                        break

                if matching_challenge is not None:
                    (res, otp_counter, reply) = \
                        self.do_request(passw, user=user)

                    ## everything is ok, we remove the challenge
                    if res is True and otp_counter >= 0:
                        matching_challenges.append(matching_challenge)

            ## in case of remote check pin, we just forward everything
            else:
                (res, otp_counter, reply) = \
                    self.do_request(passw, transactionid=transid, user=user)

        return (otp_counter, matching_challenges)

    @log_with(log)
    def checkPin(self, pin, options=None):
        """
        check the pin - either remote or localy
        - in case of remote, we return true, as the
          the splitPinPass will put the passw then in the otpVal
        """
        res = True

        ## only, if pin should be checked localy
        if self.check_pin_local():
            res = TokenClass.checkPin(self, pin)

        return res

    @log_with(log)
    def splitPinPass(self, passw):
        """
        Split the PIN and the OTP value.
        Only if it is locally checked and not remotely.

        :param passw: the password with pin and otp
        :return: tuple of the (success, pin and otpvalue)

        """
        res = 0

        local_check = self.check_pin_local()


        if local_check:
            (res, pin, otpval) = TokenClass.splitPinPass(self, passw)
        else:
            pin = ""
            otpval = passw

        return (res, pin, otpval)

###eof#########################################################################



# -*- coding: utf-8 -*-
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  License:  AGPLv3
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''
resolver handling
'''


import logging
import re
import copy

from privacyidea.lib.context import context

from privacyidea.lib.config import storeConfig
from privacyidea.lib.config import getGlobalObject
from privacyidea.lib.config import removeFromConfig
from privacyidea.lib.config import get_privacyIDEA_config

from privacyidea.lib.util import getParam
from privacyidea.lib.crypto import decryptPassword

import traceback
from privacyidea.lib.log import log_with

required = True
optional = False


__all__ = [ 'defineResolver', 'checkResolverType', 'splitResolver',
            'getResolverList', 'getResolverInfo', 'deleteResolver',
            'getResolverObject', 'initResolvers', 'closeResolvers',
            'setupResolvers'
            ]

log = logging.getLogger(__name__)


class Resolver():
    """
    helper class to define a new resolver
    """
    def __init__(self, name=None):
        self.name = name
        self.type = None
        self.data = {}
        self.types = {}
        self.desc = {}

    def getDefinition(self, param):
        self.name = getParam(param, 'resolver', required)
        return getResolverInfo(self.name)

    @log_with(log)
    def setDefinition(self, param):
        '''
            handle name
        '''
        self.name = getParam(param, 'name', required)
        # We should have no \.
        # This only leads to problems.
        nameExp = "^[A-Za-z0-9_\-]+$"
        if re.match(nameExp, self.name) is None:
            e = Exception("non conformant characters in resolver name: " + self.name + " (not in " + nameExp + ")")
            raise e

        ## handle types
        self.type = getParam(param, 'type', required)
        resolvertypes = get_resolver_types()
        if self.type not in resolvertypes:
            e = Exception("resolver type : %s not in %s" %
                          (self.type, unicode(resolvertypes)))
            raise e

        resolvers = getResolverList(filter_resolver_type=self.type)
        for resolver in resolvers:
            if self.name.lower() == resolver.lower():
                if self.name == resolver:
                    continue
                e = Exception("resolver with similar name already exists: %s" %
                              (resolver))
                raise e

        resolver_config = get_resolver_classConfig(self.type)
        if self.type in resolver_config:
            config = resolver_config.get(self.type).get('config', {})
        else:
            config = resolver_config

        for k in param:
            if k != 'name' and k != 'type':
                if k.startswith('type.') == True:
                    key = k[len('type.'):]
                    self.types[key] = param.get(k)
                elif k.startswith('desc.') == True:
                    key = k[len('desc.'):]
                    self.desc[key] = param.get(k)

                elif 'session' == k:
                    ## supress session parameter
                    pass
                else:
                    self.data[k] = param.get(k)
                    if k in config:
                        self.types[k] = config.get(k)
                    else:
                        log.warn("the passed key %r is not a "
                                 "parameter for the resolver %r" % (k, self.type))
        ## now check if we have for every type def an parameter
        ok = self._sanityCheck()
        if ok != True:
            raise Exception("type definition does not match parameter! %s"
                            % unicode(param))

        return

    def _sanityCheck(self):
        ret = True
        for t in self.types:
            if self.data.has_key(t) == False:
                ret = False
        for t in self.desc:
            if self.data.has_key(t) == False:
                ret = False

        return ret

    @log_with(log)
    def saveConfig(self):
        res = 'success'
        if self.name is None:
            return "no resolver name defined"
        # do the setConfig()'s
        prefix = self.type + "."
        postfix = "." + self.name

        for d in self.data:
            key = prefix + d + postfix
            val = self.data.get(d)
            typ = None
            desc = None
            if self.types.has_key(d) == True:
                typ = self.types.get(d)

            if self.desc.has_key(d) == True:
                desc = self.desc.get(d)

            res = storeConfig(key, val, typ, desc)

        return res

@log_with(log)
def defineResolver(params):
    """
    set up a new resolver from request parameters

    :param params: dict of request parameters
    """
    resolver = Resolver()
    resolver.setDefinition(params)
    res = resolver.saveConfig()
    return res


@log_with(log)
def checkResolverType(resolver):
    """
    check if a resolver of the given type exists
    :param resolver: full qualified resolver name
                     or optional with trailing conf like:
                       useridresolver.PasswdIdResolver.IdResolver.etc_resl
    :return: Tuple of bool (True|False) and resolver
    """
    res = ""
    ret = False

    ## prepare
    reso = resolver.strip()
    reso = reso.replace("\"", "")

    ## the fully qualified resolver
    if reso in context.resolver_clazzes:
        res = context.resolver_clazzes.get(reso)
        ret = True
    else:
        ## if the last argument is the configuration
        pack = reso.split('.')
        rtype = ".".join(pack[:-1])
        conf = pack[-1]

        ## lookup, if there is a resolver definition
        if rtype in context.resolver_types:
            res = "%s.%s" % (rtype, conf)
            ret = True
        ##
        else:
            ## legacy support, where resolver is defined as
            #    "useridresolver.passwdresolver.mrealm"
            # so we only could rely only on the type definition e.g.
            #  'passwdresolver' as part of the string
            for res_id, res_type in context.resolver_types.iteritems():
                if res_type in reso:
                    res = "%s.%s" % (res_id, conf)
                    ret = True
                    break

    ##  is resolver defined in the privacyidea config
    try:
        getResolverObject(res)
    except Exception as exx:
        log.error("Failed to setup resolver %r: %r" % (res, exx))
        log.error("%r" % traceback.format_exc())
        # upper layer (controller) will catch
        raise(exx)
        # res = False
        # ret = False

    return (ret, res)

#### helper functions to retrieve information from the UserIDResolvers ###################
@log_with(log)
def splitResolver(resolver):

    reso = resolver.strip()
    reso = reso.replace("\"", "")
    # old lin-otp had only 3 parts.
    # new lin-otp had 4 parts. 
    # we break compatibility and have more than 4 parts.    
    try:
        l = reso.rsplit('.',3)
        package = l[0]
        module = l[1]
        class_ = l[2]
        conf = l[3]
    except Exception as e:
        log.error("split of resolver failed %s : %r " % (reso, e))
        raise Exception("invalid resolver class specification" + reso)
    return (package, module, class_, conf)

## external system/getResolvers
@log_with(log)
def getResolverList(filter_resolver_type=None):
    '''
    Gets the list of configured resolvers

    :param filter_resolver_type: Only resolvers of the given type are returned
    :type filter_resolver_type: string
    :rtype: Dictionary of the resolvers and their configuration
    '''
    Resolvers = {}
    resolvertypes = get_resolver_types()

    conf = get_privacyIDEA_config()
    for entry in conf:

        for typ in resolvertypes:
            if entry.startswith("privacyidea." + typ):
                #the realm might contain dots "."
                # so take all after the 3rd dot for realm
                r = {}
                resolver = entry.split(".", 3)
                # An old entry without resolver name
                if len(resolver) <= 3:
                    break
                r["resolvername"] = resolver[3]
                r["entry"] = entry
                r["type"] = typ

                if (filter_resolver_type is None) or (filter_resolver_type and filter_resolver_type == typ):
                    Resolvers[resolver[3]] = r
                # Dont check the other resolver types
                break

    return Resolvers


@log_with(log)
def getResolverInfo(resolvername):
    '''
    return the resolver info of the given resolvername

    :param resolvername: the requested resolver
    :type  resolvername: string

    :return : dict of resolver description
    '''
    resolver_dict = {}
    typ = ""
    resolvertypes = get_resolver_types()

    descr = {}

    conf = get_privacyIDEA_config()

    for entry in conf:

        for typ in resolvertypes:

            ## get the typed values of the descriptor!
            resolver_conf = get_resolver_classConfig(typ)
            if typ in resolver_conf:
                descr = resolver_conf.get(typ).get('config', {})
            else:
                descr = resolver_conf

            if entry.startswith("privacyidea." + typ) and entry.endswith(resolvername):
                #the realm might contain dots "."
                # so take all after the 3rd dot for realm
                resolver = entry.split(".", 3)
                # An old entry without resolver name
                if len(resolver) <= 3:
                    break

                value = conf.get(entry)
                if resolver[2] in descr:
                    configEntry = resolver[2]
                    if descr.get(configEntry) == 'password':

                        ## do we already have the decrypted pass?
                        if 'enc' + entry in conf:
                            value = conf.get('enc' + entry)
                        else:
                            ## if no, we take the encpass and decrypt it
                            value = conf.get(entry)
                            try:
                                en = decryptPassword(value)
                                value = en
                            except:
                                log.info("Decryption of resolver passwd failed: compatibility issue?")

                resolver_dict[ resolver[2] ] = value
                # Dont check the other resolver types

                break

    return { "type" : typ, "data" : resolver_dict, "resolver" : resolvername}

@log_with(log)
def deleteResolver(resolvername):
    '''
    delete a resolver and all related config entries

    :paramm resolvername: the name of the to be deleted resolver
    :type   resolvername: string
    :return: sucess or fail
    :rtype:  boelean

    '''
    res = False

    resolvertypes = get_resolver_types()
    conf = get_privacyIDEA_config()

    delEntries = []

    for entry in conf:
        rest = entry.split(".", 3)
        lSplit = len(rest)
        if lSplit > 3:
            rConf = rest[lSplit - 1]
            if rConf == resolvername:
                if rest[0] == "privacyidea" or rest[0] == "encprivacyidea" :
                    typ = rest[1]
                    if typ in resolvertypes:
                        delEntries.append(entry)

    if len(delEntries) > 0 :
        try:
            for entry in delEntries:
                res = removeFromConfig(entry)
                log.debug("removing key: %s" % entry)
                res = True
        except Exception as e:
            log.error("deleteResolver: %r" % e)
            res = False


    return res




## external in token.py user.py validate.py
@log_with(log)
def getResolverObject(resolvername):
    """
    get the resolver instance from a resolver name spec
    - either take the class from the request context
    - or create one from the global object list + init with resolver config

    :remark: the resolver object is preserved in the request context, so that
              a resolver could preserve a connection durung a request

    :param resolvername: the resolver string as from the token including
                         the config as last part
    :return: instance of the resolver with the loaded config

    """
    r_obj = None
    ### this patch is a bit hacky:
    ## the normal request has a request context, where it retrieves
    ## the resolver info from and preserves the loaded resolvers for reusage
    ## But in case of a authentication request (by a redirect from a 401)
    ## the caller is no std request and the context object is missing :-(
    ## The solution is to deal with local references, either to the
    ## global context or to local data (where we have no reuse of the resolver)

    resolvers_loaded = {}
    r_obj_class = None
    try:
        if hasattr(context, 'resolvers_loaded') == False:
            setattr(context, 'resolvers_loaded', {})
        resolvers_loaded = context.resolvers_loaded
    except Exception as exx:
        resolvers_loaded = {}

    ## test if there is already a resolver of this kind loaded
    if resolvername in resolvers_loaded:
        return resolvers_loaded.get(resolvername)

    ## no resolver - so instantiate one
    else:
        parts = resolvername.split('.')
        if len(parts) > 2:
            re_name = '.'.join(parts[:-1])
            r_obj_class = get_resolver_class(re_name)

        if r_obj_class is None:
            log.error("unknown resolver class %s " % resolvername)
            return r_obj

        ## create the resolver instance and load the config
        r_obj = r_obj_class()
        conf = resolvername.split(".")[-1]

        if r_obj is not None:
            config = get_privacyIDEA_config()
            r_obj.loadConfig(config, conf)
            resolvers_loaded[resolvername] = r_obj

    return r_obj

## external lib/base.py
@log_with(log)
def setupResolvers(config=None, cache_dir="/tmp"):
    """
    hook for the server start -
        initialize the resolvers
    """
    glo = getGlobalObject()

    resolver_clazzes = copy.deepcopy(glo.getResolverClasses())
    for resolver_clazz in resolver_clazzes.values():
        if hasattr(resolver_clazz, 'setup'):
            try:
                resolver_clazz.setup(config=config, cache_dir=cache_dir)
            except Exception as exx:
                log.error("failed to call setup of %r" % resolver_clazz)

    return

@log_with(log)
def initResolvers():
    """
    hook for the request start -
        create  a deep copy of the dict with the global resolver classes
    """
    try:
        glo = getGlobalObject()

        resolver_clazzes = copy.deepcopy(glo.getResolverClasses())
        setattr(context, 'resolver_clazzes', resolver_clazzes)

        resolver_types = copy.deepcopy(glo.getResolverTypes())
        setattr(context, 'resolver_types', resolver_types)

        ## dict of all resolvers, which are instatiated during the request
        setattr(context, 'resolvers_loaded', {})

    except Exception as exx:
        log.error("Failed to initialize resolver in context %r" % exx)
    return

## external lib/base.py
@log_with(log)
def closeResolvers():
    """
    hook to close the resolvers at the end of the request
    """

    if hasattr(context, 'resolvers_loaded'):
        try:
            for resolver in context.resolvers_loaded.values():
                if hasattr(resolver, 'close'):
                    resolver.close()

        except Exception as exx:
            log.error("Failed to close resolver in context %r" % exx)
    return


## internal functions
@log_with(log)
def get_resolver_class(resolver_type):
    '''
    return the class object for a resolver type
    :param resolver_type: string specifying the resolver
                          fully qualified or abreviated
    :return: resolver object class
    '''
    ret = None

    ### this patch is a bit hacky:
    ## the normal request has a request context, where it retrieves
    ## the resolver info from and preserves the loaded resolvers for reusage
    ## But in case of a authentication request (by a redirect from a 401)
    ## the caller is no std request and the context object is missing :-(
    ## The solution is, to deal with local references, either to the
    ## global context or to local data

    try:
        resolver_clazzes = context.resolver_clazzes
        resolver_types = context.resolver_types
    except Exception as exx:
        glo = getGlobalObject()
        resolver_clazzes = copy.deepcopy(glo.getResolverClasses())
        resolver_types = copy.deepcopy(glo.getResolverTypes())

    parts = resolver_type.split('.')
    ## resolver is fully qualified
    if len(parts) > 1:
        if resolver_type in resolver_clazzes:
            ret = resolver_clazzes.get(resolver_type)

    ## resolver is in abreviated form, we have to do a reverse lookup
    elif resolver_type in resolver_types.values():
        for k, v in resolver_types.iteritems():
            if v == resolver_type:
                ret = resolver_clazzes.get(k, None)
                break
    if ret is None:
        pass
    return ret


def get_resolver_types():
    """
    get the array of the registred resolvers

    :return: array of resolvertypes like 'passwdresolver'
    """
    return context.resolver_types.values()


@log_with(log)
def get_resolver_classConfig(claszzesType):
    """
    get the configuration description of a resolver

    :param claszzesType: literal resolver type
    :return: configuration description dict
    """
    descriptor = None
    resolver_class = get_resolver_class(claszzesType)

    if resolver_class is not None:
        descriptor = resolver_class.getResolverClassDescriptor()

    return descriptor

def get_resolver_name(idRes):
    '''
    :param idRes: The id of a resolver in the dotted notation
    :return: the name of the resolver
    
    The name of the resolver is always the last part
    
    privacyidea.passwdidresolver.filename.reso1 will return "reso1"
    '''
    parts = idRes.split('.')
    return parts[-1]


#eof###########################################################################




# -*- coding: utf-8 -*-
#
#  privacyIDEA
#  Jul 18, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
from privacyidea.lib.applications import MachineApplicationBase
from privacyidea.lib.user import getUserInfo
import logging
log = logging.getLogger(__name__)
# from privacyidea.lib.log import log_with
from privacyidea.lib.token import getTokens4UserOrSerial


class MachineApplication(MachineApplicationBase):
    '''
    This is the application for SSH.
    
    Possible options:
        option_user
    
    '''
    application_name = "ssh"
    '''as the autentication item is no sensitive information,
    we can set bulk_call to True. Thus the admin can call
    all public keys to distribute them via salt.
    FIXME: THis is only true for SSH pub keys.
    If we would support OTP with SSH, this might be sensitive information!
    '''
    allow_bulk_call = True
    
    def get_authentication_item(self,
                                token_type,
                                serial,
                                challenge=None):
        '''
        :param token_type: the type of the token. At the moment
                           we support the tokenype "sshkey"
        :param serial:     the serial number of the token.
        :return auth_item: For Yubikey token type it
                           returns a dictionary with a "challenge" and
                           a "response".
        '''
        ret = {}
        if (token_type.lower() == "sshkey"):
                toks = getTokens4UserOrSerial(serial=serial)
                if len(toks) == 1:
                    # tokenclass is a SSHkeyTokenClass
                    tokclass = toks[0]
                    # We just return the ssh public key, so that
                    # it can be included into authorized keys.
                    ret["sshkey"] = tokclass.get_sshkey()
                    # We return the username if the token is assigned to a
                    # user, so that this username could be used to save
                    # the ssh key accordingly
                    (uid, resolver, resClass) = toks[0].getUser()
                    uInfo = getUserInfo(uid, resolver, resClass)
                    if "username" in uInfo:
                        ret["username"] = uInfo.get("username")
                    # ret["info"] = uInfo
                    
        else:
                log.info("Token %r, type %r is not supported by"
                         "SSH application module" % (serial, token_type))
            
        return ret
    
    def get_options(self):
        '''
        returns a dictionary with a list of required and optional options
        '''
        return {'required': [],
                'optional': ['option_user']}


#!/usr/bin/python
# -*- coding: utf-8 -*-
#  copyright 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
# This code is free software; you can redistribute it and/or
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
import os
import time
from stat import ST_SIZE, ST_MTIME
from subprocess import call
import ConfigParser
import StringIO
import re
import sys
from privacyidea.lib.ext import nginxparser
from privacyidea.lib.freeradiusparser import ClientConfParser
from privacyidea.lib.cronjobparser import CronJobParser
import crypt
import random
import fileinput
import socket
from subprocess import Popen, PIPE
from privacyidea.lib.util import generate_password

DATABASE = "privacyidea"
DBUSER = "privacyidea"
POOL = "./0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
CRONTAB = "/etc/crontab"
CRON_USER = "privacyidea"
BACKUP_CMD = "/usr/bin/privacyidea-backup"
RESTORE_CMD = "/usr/bin/privacyidea-restore %s"
DEFAULT_CONFIG = "/etc/privacyidea/privacyidea.ini"
BACKUP_DIR = "/var/lib/privacyidea/backup"


class Backup(object):
    
    def __init__(self,
                 config_dir="/etc/privacyidea/backup",
                 data_dir="/var/lib/privacyidea/backup"):
        self.data_dir = data_dir
        self.assignments = {}
        self.cronjobs = {}
        self.CP = CronJobParser()

    def backup_now(self, password=None):
        '''
        Create a backup of the system right now
        The current backup will contain the
        encryption key. This will be encrypted with
        the password.
        '''
        call(BACKUP_CMD, shell=True)
        
    def restore_backup(self, bfile, password=None):
        '''
        Restore the backup file.
        
        :param bfile: the tgz file name without the path
        :type bfile: string
        '''
        call(RESTORE_CMD % BACKUP_DIR + "/" + bfile, shell=True)
    
    def get_cronjobs(self):
        '''
        Parse the cronjob and return the backup times
        '''
        config = self.CP.get_dict()
        self.assignments = config.get("assignments")
        self.cronjobs = config.get("cronjobs")
        return self.cronjobs
    
    def add_backup_time(self, dc):
        '''
        Add a backup time to the cronjobs
        
        :param dc: Date component of minute, hour, dom, month, dow
        :type dc: list
        '''
        self.get_cronjobs()
        self.cronjobs.append({"minute": dc[0],
                              "hour": dc[1],
                              "dom": dc[2],
                              "month": dc[3],
                              "dow": dc[4],
                              "user": CRON_USER,
                              "command": BACKUP_CMD})
        self.CP.save({"assignments": self.assignments,
                      "cronjobs": self.cronjobs},
                     CRONTAB)
        
    def get_backups(self):
        '''
        List the available backups in the self.data_dir
        
        :return: dict of backups. Key is the filename, and
                 "size" and "time"
        '''
        backups = {}
        try:
            allfiles = os.listdir(self.data_dir)
        except OSError:
            return backups
        
        for f in allfiles:
            if f.startswith("privacyidea-backup"):
                st = os.stat(self.data_dir + "/" + f)
                size = "%iMB" % (int(st[ST_SIZE]) / (1024 * 1024))
                mtime = time.asctime(time.localtime(st[ST_MTIME]))
                backups[f] = {"size": size,
                              "time": mtime}
        return backups
    
    def del_backup_time(self, hour, minute, month, dom, dow):
        '''
        Delete a backup time from the cronjob
        '''
        self.get_cronjobs()
        jobs_num = len(self.cronjobs)
        i = jobs_num - 1
        while i >= 0:
            cronjob = self.cronjobs[i]
            if (cronjob.get("hour") == hour and
                cronjob.get("minute") == minute and
                cronjob.get("dom") == dom and
                cronjob.get("month") == month and
                cronjob.get("dow") == dow and
                cronjob.get("user") == CRON_USER):
                self.cronjobs.pop(i)
            i -= 1
            
        if len(self.cronjobs) != jobs_num:
            self.CP.save({"assignments": self.assignments,
                          "cronjobs": self.cronjobs},
                         CRONTAB)


class PrivacyIDEAConfig(object):
    
    ini_template = """[DEFAULT]
debug = false
profile = false
smtp_server = localhost
error_email_from = paste@localhost
privacyideaAudit.type = privacyidea.lib.auditmodules.sqlaudit
privacyideaAudit.key.private = %(here)s/private.pem
privacyideaAudit.key.public = %(here)s/public.pem
privacyideaAudit.sql.highwatermark = 10000
privacyideaGetotp.active = False
privacyideaSecretFile = %(here)s/encKey
privacyideaSuperuserFile = %(here)s/admin-users
privacyideaSuperuserRealms = superuser, 2ndsuperusers
privacyIDEASessionTimeout = 1200
privacyideaURL = https://localhost
privacyideaURL.disable_ssl = False
radius.dictfile = %(here)s/dictionary
radius.nas_identifier = privacyIDEA
privacyideaMachine.applications = privacyidea.lib.applications.ssh, privacyidea.lib.applications.luks

[server:main]
use = egg:Paste#http
host = 0.0.0.0
port = 5001

[app:main]
use = egg:privacyIDEA
sqlalchemy.url = sqlite:///%(here)s/token.sqlite
sqlalchemy.pool_recycle = 3600
full_stack = true
static_files = true
who.log_level = debug
who.log_file = %(here)s/privacyidea.log
cache_dir = %(here)s/data
custom_templates = %(here)s/custom-templates/

[loggers]
keys = root, privacyidea, sqlalchemy

[logger_root]
level = WARNING
handlers = file

[logger_privacyidea]
level = INFO
handlers = file
qualname = privacyidea

[logger_sqlalchemy]
level = ERROR
handlers = file
qualname = sqlalchemy.engine

[handlers]
keys = file

[handler_file]
class = handlers.RotatingFileHandler
args = ('/var/log/privacyidea/privacyidea.log','a', 10000000, 4)
level = INFO
formatter = generic

[formatters]
keys = generic

[formatter_generic]
class = privacyidea.lib.log.SecureFormatter
format = %(asctime)s %(levelname)-5.5s {%(thread)d} [%(name)s][%(funcName)s #%(lineno)d] %(message)s
datefmt = %Y/%m/%d - %H:%M:%S
"""
    
    def __init__(self,
                 file="/etc/privacyidea/privacyidea.ini",
                 init=False):
        self.file = file
        self.raw_config = ConfigParser.RawConfigParser()
        self.raw_config.optionxform = str
        if init:
            self.raw_config.readfp(StringIO.StringIO(self.ini_template))
        else:
            self.raw_config.read(self.file)
            
        self.config = ConfigParser.ConfigParser()
        self.config.optionxform = str
        if init:
            self.config.readfp(StringIO.StringIO(self.ini_template))
        else:
            self.config.read(self.file)
            
        config_path = os.path.abspath(os.path.dirname(self.file))
        self.config.set("DEFAULT", "here", config_path)

    def initialize(self):
        '''
        Initialize the ini file
        '''
        self.raw_config.readfp(StringIO.StringIO(self.ini_template))
    
    def get_getotp(self):
        getotp = self.raw_config.get("DEFAULT", "privacyideaGetotp.active")
        # might be bool or string
        if type(getotp) == bool:
            return getotp
        return getotp.upper() == "TRUE"
    
    def set_getotp(self, bAllow=False):
        self.raw_config.set("DEFAULT", "privacyideaGetotp.active", bAllow)
    
    def get_keyfile(self):
        return self.raw_config.get("DEFAULT", "privacyideaSecretFile")

    def get_adminfile(self):
        return self.raw_config.get("DEFAULT", "privacyideaSuperuserFile")

    def set_adminfile(self, afile):
        self.raw_config.set("DEFAULT", "privacyideaSuperuserFile", afile)
        
    def get_admins(self):
        '''
        return a list of admins
        '''
        admins = []
        admin_file = self.config.get("DEFAULT", "privacyideaSuperuserFile")
        
        try:
            f = open(admin_file, "r")
        except:
            f = None
        
        if f:
            for line in f:
                try:
                    admins.append((line.split(":")[0], line.split(":")[4]))
                except IndexError:
                    pass
            f.close()
        
        return admins
        
    def set_admin(self,
                  admin,
                  password,
                  description=""):
        '''
        create a new admin
        '''
        admin_file = self.config.get("DEFAULT", "privacyideaSuperuserFile")
        try:
            f = open(admin_file, "r")
        except:
            f = None
        max_id = 0
        
        if f:
            for line in f:
                try:
                    uid = int(line.split(":")[2])
                    if uid > max_id:
                        max_id = uid
                except IndexError:
                    pass
            f.close()
            
        salt = (POOL[random.randrange(0, len(POOL))]
                + POOL[random.randrange(0, len(POOL))])
        encryptedPW = crypt.crypt(password, salt)
        gid = max_id + 1
        home = ""
        shell = ""
        
        f = open(admin_file, "a")
        f.write("%s:%s:%s:%s:%s:%s:%s" % (admin,
                                          encryptedPW,
                                          gid,
                                          gid,
                                          description,
                                          home,
                                          shell))
        f.close()
        
    def update_admin(self,
                     admin,
                     password,
                     description=""):
        '''
        update an existing admin
        '''
        admin_file = self.config.get("DEFAULT", "privacyideaSuperuserFile")
        for line in fileinput.input(admin_file, inplace=True):
            # admin_in_file, encryptedPW, uid,
            # gid, description_in_file,
            # home, shell
            fields = line.split(":")
            if admin == fields[0]:
                description_in_file = fields[4]
                salt = (POOL[random.randrange(0, len(POOL))]
                        + POOL[random.randrange(0, len(POOL))])
                encryptedPW = crypt.crypt(password, salt)
                if description:
                    description_in_file = description
                print "%s:%s:%s:%s:%s:%s:%s" % (admin,
                                                encryptedPW,
                                                fields[2],
                                                fields[3],
                                                description_in_file,
                                                fields[5],
                                                fields[6])
            else:
                print line,
                        
    def delete_admin(self, admin):
        '''
        delete the administrator
        '''
        admin_file = self.config.get("DEFAULT", "privacyideaSuperuserFile")
        for line in fileinput.input(admin_file, inplace=True):
            admin_in_file = line.split(":")[0]
            if admin == admin_in_file:
                continue
            print line,
    
    def get_adminrealms(self):
        return self.raw_config.get("DEFAULT", "privacyideaSuperuserRealms")

    def set_adminrealms(self, arealms):
        self.raw_config.set("DEFAULT", "privacyideaSuperuserRealms", arealms)

    def get_DB(self):
        return self.raw_config.get("app:main", "sqlalchemy.url")
    
    def get_DB_dict(self):
        res = {}
        connect_string = self.get_DB()
        pattern = "^(.*)://(.*)/(.*)$"
        m = re.match(pattern, connect_string)
        if m:
            hostpart = m.group(2)
            dbtype = m.group(1)
            dbname = m.group(3)
            password = None
            user = None
            hostparts = hostpart.split('@')
            if len(hostparts) == 2:
                host = hostparts[1]
                try:
                    user, password = hostparts[0].split(':')
                except:
                    user = hostparts[0]
                    password = ""
                
            elif len(hostparts) == 1:
                host = hostparts[0]

            return {"host": host,
                    "user": user,
                    "password": password,
                    "type": dbtype,
                    "database": dbname}

        return res
   
    def DB_init(self):
        '''
        run the paster setup-app command
        '''
        print "Running paster setup-up with %s" % self.file
        p = Popen(['paster', 'setup-app', self.file],
                  stdin=PIPE,
                  stdout=PIPE,
                  stderr=PIPE)
        output, err = p.communicate()
        r = p.returncode
        if r == 0:
            print "Created database tables"
            return True, output
        
        return False, output + "\n" + err
    
    def DB_create(self, database=DATABASE, user=DBUSER):
        '''
        Create the database locally and setup the user.
        
        execute the commands:
        mysqladmin --defaults-file=/etc/mysql/debian.cnf create TEST
        echo  'grant all privileges on TEST.* to "p"@"localhost" \
               identified by "test";'  | mysql \
               --defaults-file=/etc/mysql/debian.cnf
 
        :return (database, user, password): returns the tuple of the
                                  database name,
                                  the username and the password.
        '''
        password = generate_password(size=30)
        dbconfig = (database, user, password)
        
        p = Popen(["mysqladmin",
                   "--defaults-file=/etc/mysql/debian.cnf",
                   "create",
                   database],
                  stdin=PIPE,
                  stdout=PIPE,
                  stderr=PIPE)
        
        _output, err = p.communicate()
        r = p.returncode
        if r != 0 and "database exists" not in err:
            return False, ("%s, %s" % (err, r))
              
        p = Popen(["mysql",
                   "--defaults-file=/etc/mysql/debian.cnf"],
                  stdin=PIPE,
                  stdout=PIPE,
                  stderr=PIPE)
        
        _output, err = p.communicate(input='grant all privileges on %s.* to '
                                           '"%s"@"localhost" '
                                           'identified by "%s";' % (database,
                                                                    user,
                                                                    password),)
        r = p.returncode
        if r != 0:
            return False, (err)
        
        return True, (dbconfig)
        
    def set_DB(self, db_config):
        """
        :param db_config: sets the database parameters (type, host, user,
                          password, database)
        :type db_config: dictionary
        """
        db_str = self.raw_config.get("app:main", "sqlalchemy.url")
        if not db_str:
            db_str = "mysql://privacyidea:privacyidea@localhost/privacyidea"
        print db_config
        # compile hostname
        DB_dict = self.get_DB_dict()
        host = DB_dict.get("host")
        user = DB_dict.get("user")
        dbtype = DB_dict.get("type")
        dbname = DB_dict.get("database")
        password = DB_dict.get("password")

        if db_config.get("type"):
            dbtype = db_config.get("type")
        if db_config.get("database"):
            dbname = db_config.get("database")
        if db_config.get("password") is not None:
            password = db_config.get("password")
        if db_config.get("user") is not None:
            user = db_config.get("user")
        if db_config.get("host"):
            host = db_config.get("host")
            
        if password:
            hostpart = "%s:%s@%s" % (user, password, host)
        elif user:
            hostpart = "%s@%s" % (user, host)
        else:
            hostpart = host
        db_str = "%s://%s/%s" % (dbtype, hostpart, dbname)
            
        self.raw_config.set("app:main", "sqlalchemy.url", db_str)
     
    def get_loglevel(self):
        return self.raw_config.get("handler_file", "level")
    
    def set_loglevel(self, level):
        self.raw_config.set("handler_file", "level", level)
        
    def toggle_getotp(self):
        getotp = self.get_getotp()
        self.raw_config.set("DEFAULT", "privacyideaGetotp.active", not getotp)
        
    def create_audit_keys(self):
        # We can not use the RawConfigParser, since it does not
        # replace the (here)s statement
        private = self.config.get("DEFAULT", "privacyideaAudit.key.private")
        public = self.config.get('DEFAULT', "privacyideaAudit.key.public")

        print "Create private key %s" % private
        r = call("openssl genrsa -out %s 2048" % private,
                 shell=True)
        if r == 0:
            print "create private key: %s" % private

        print "Create public key %s" % public
        r = call("openssl rsa -in %s -pubout -out %s" % (private, public),
                 shell=True)
        if r == 0:
            print "written public key: %s" % private
            return True, private
        
        return False, private
    
    def create_encryption_key(self):
        # We can not use the RawConfigParser, since it does not
        # replace the (here)s statement
        enckey = self.config.get('DEFAULT', "privacyideaSecretFile")

        r = call("dd if=/dev/urandom of='%s' bs=1 count=96" % enckey,
                 shell=True)
        if r == 0:
            print "written enckey: %s" % enckey
            return True, enckey
        return False, enckey

    def save(self):
        with open(self.file, 'wb') as configfile:
            self.raw_config.write(configfile)
        print "Config file %s saved." % self.file


class NginxConfig(object):
    
    NGINX = 0
    UWSGI = 1
    default_file = ["privacyidea", "privacyidea.xml"]
    default_dir_enabled = ["/etc/nginx/sites-enabled",
                           "/etc/uwsgi/apps-enabled"]
    default_dir_available = ["/etc/nginx/sites-available",
                             "/etc/uwsgi/apps-available"]
    
    def __init__(self, files=None):
        '''
        :param files: The default config files for nginx and uwsgi
        :type files: list of two files
        '''
        if files is None:
            files = self.default_file
        self.configfile = files

    def is_active(self):
        '''
        :return: A list of boolean indicating if nginx and uwsgi are active
        '''
        r1 = os.path.isfile(self.default_dir_enabled[0] + "/" +
                            self.configfile[0])
        r2 = os.path.isfile(self.default_dir_enabled[1] + "/" +
                            self.configfile[1])
        return r1, r2
    
    def get(self):
        config = nginxparser.load(open(self.default_dir_available[self.NGINX]
                                       + "/" +
                                       self.configfile[self.NGINX]))
        return config
    
    def enable(self):
        for i in [self.NGINX, self.UWSGI]:
            if not os.path.exists(self.default_dir_enabled[i]):
                os.mkdir(self.default_dir_enabled[i])

            if not os.path.exists(self.default_dir_enabled[i] +
                                  "/" + self.configfile[i]):
                os.symlink(self.default_dir_available[i] +
                           "/" + self.configfile[i],
                           self.default_dir_enabled[i] +
                           "/" + self.configfile[i])
        return
    
    def enable_webservice(self, webservices):
        """
        :param webservices: list of activated links
        :type webservices: list
        """
        if not os.path.exists(self.default_dir_enabled[self.NGINX]):
            os.mkdir(self.default_dir_enabled[self.NGINX])

        active_list = os.listdir(self.default_dir_enabled[self.NGINX])
        # deactivate services
        for service in active_list:
            if service not in webservices:
                # disable webservice
                os.unlink(self.default_dir_enabled[self.NGINX] +
                          "/" + service)
        # activate services
        for service in webservices:
            # enable webservice
            if not os.path.exists(self.default_dir_enabled[self.NGINX] +
                                  "/" + service):
                os.symlink(self.default_dir_available[self.NGINX] +
                           "/" + service,
                           self.default_dir_enabled[self.NGINX] +
                           "/" + service)
    
    def get_webservices(self):
        '''
        returns the contents of /etc/nginx/sites-available
        '''
        ret = []
        file_list = os.listdir(self.default_dir_available[self.NGINX])
        active_list = os.listdir(self.default_dir_enabled[self.NGINX])
        for k in file_list:
            if k in active_list:
                ret.append((k, "", 1))
            else:
                ret.append((k, "", 0))
        return ret
    
    def disable(self):
        for i in [self.NGINX, self.UWSGI]:
            os.unlink(self.default_dir_enabled[i] + "/" + self.configfile[i])
        return
    
    def _get_val(self, data, key):
        '''
        returns a value for a given key from a list of tuples.
        '''
        for kv in data:
            if kv[0] == key:
                return kv[1]
        return
    
    def get_certificates(self):
        '''
        return a tuple of the certificate and the private key
        '''
        config = self.get()
        cert = None
        key = None
        for server in config:
            if server[0] == ["server"]:
                # server config
                if self._get_val(server[1], "listen")[-3:].lower() == "ssl":
                    # the ssl config
                    cert = self._get_val(server[1], "ssl_certificate")
                    key = self._get_val(server[1], "ssl_certificate_key")
        return cert, key
    
    def create_certificates(self):
        certificates = self.get_certificates()
        hostname = socket.getfqdn()
        print("Generating SSL certificate %s and key %s" % certificates)
        if certificates[0] and certificates[1]:
            command = ("openssl req -x509 -newkey rsa:2048 -keyout %s -out"
                       " %s -days 1000 -subj /CN=%s -nodes" %
                       (certificates[1],
                        certificates[0],
                        hostname))
            r = call(command, shell=True)
            if r == 0:
                print "Created the certificate and the key."
                os.chmod(certificates[1], 0x400)
            else:
                print "Failed to create key and certificate: %i" % r
                sys.exit(r)
                
    def restart(self,
                service,
                do_print=False):
        '''
        Restart the nginx and uwsgi
        '''
        p = Popen(['service',
                   service,
                   'restart'],
                  stdin=PIPE,
                  stdout=PIPE,
                  stderr=PIPE)
        _output, _err = p.communicate()
        r = p.returncode
        if r == 0:
            if do_print:
                print "Service %s restarted" % service
        else:
            if do_print:
                print _err


class FreeRADIUSConfig(object):
       
    def __init__(self, client="/etc/freeradius/clients.conf"):
        '''
        Clients are always kept persistent on the file system
        :param client: clients.conf file.
        '''
        # clients
        self.ccp = ClientConfParser(infile=client)
        self.config_path = os.path.dirname(client)
        self.dir_enabled = self.config_path + "/sites-enabled"
        self.dir_available = self.config_path + "/sites-available"
        
    def clients_get(self):
        clients = self.ccp.get_dict()
        return clients
    
    def client_add(self, client=None):
        '''
        :param client: dictionary with a key as the client name and attributes
        :type client: dict
        '''
        if client:
            clients = self.clients_get()
            for client, attributes in client.iteritems():
                clients[client] = attributes
            
            self.ccp.save(clients)
        
    def client_delete(self, clientname=None):
        '''
        :param clientname: name of the client to be deleted
        :type clientname: string
        '''
        if clientname:
            clients = self.clients_get()
            clients.pop(clientname, None)
            self.ccp.save(clients)

    def set_module_perl(self):
        '''
        Set the perl module
        '''
        f = open(self.config_path + "/modules/perl", "w")
        f.write("""perl {
        module = /usr/share/privacyidea/freeradius/privacyidea_radius.pm
}
        """)
        
    def enable_sites(self, sites):
        """
        :param sites: list of activated links
        :type sitess: list
        """
        if not os.path.exists(self.dir_enabled):
            os.mkdir(self.dir_enabled)

        active_list = os.listdir(self.dir_enabled)
        # deactivate site
        for site in active_list:
            if site not in sites:
                # disable site
                os.unlink(self.dir_enabled +
                          "/" + site)
        # activate site
        for site in sites:
            # enable site
            if not os.path.exists(self.dir_enabled +
                                  "/" + site):
                os.symlink(self.dir_available +
                           "/" + site,
                           self.dir_enabled +
                           "/" + site)
    
    def get_sites(self):
        '''
        returns the contents of /etc/freeradius/sites-available
        '''
        ret = []
        file_list = os.listdir(self.dir_available)
        active_list = os.listdir(self.dir_enabled)
        for k in file_list:
            if k in active_list:
                ret.append((k, "", 1))
            else:
                ret.append((k, "", 0))
        return ret
        
#
#  users:
#     DEFAULT Auth-Type := perl
#
#
#



# -*- coding: utf-8 -*-
#
#  privacyIDEA is a fork of LinOTP
#  May 08, 2014 Cornelius Kölbel
#  License:  AGPLv3
#  contact:  http://www.privacyidea.org
#
#  2014-10-03 Add getInitDetail
#             Cornelius Kölbel <cornelius@privacyidea.org>
#
#  Copyright (C) 2010 - 2014 LSE Leading Security Experts GmbH
#  contact:  http://www.linotp.org
#            http://www.lsexperts.de
#            linotp@lsexperts.de
#
# This code is free software; you can redistribute it and/or
# modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
# License as published by the Free Software Foundation; either
# version 3 of the License, or any later version.
#
# This code is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU AFFERO GENERAL PUBLIC LICENSE for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
'''
  Description:  This file containes the dynamic hmac token implementation:
              - HmacTokenClas   (HOTP)

  Dependencies: -

'''

import time
from datetime import datetime

from privacyidea.lib.HMAC import HmacOtp
from privacyidea.lib.util import getParam
from privacyidea.lib.config import getFromConfig
from privacyidea.lib.tokenclass import TokenClass

from privacyidea.lib.validate import check_pin
from privacyidea.lib.validate import check_otp
from privacyidea.lib.log import log_with
from privacyidea.lib.apps import create_google_authenticator_url as cr_google
from privacyidea.lib.apps import create_oathtoken_url as cr_oath
from privacyidea.lib.reply import create_img

optional = True
required = False

from pylons.i18n.translation import _

import traceback
import logging
log = logging.getLogger(__name__)

keylen = {'sha1': 20,
          'sha256': 32,
          'sha512': 64
          }


class HmacTokenClass(TokenClass):
    '''
    hotp token class implementation
    '''

    @classmethod
    def getClassType(cls):
        '''
        getClassType - return the token type shortname

        :return: 'hmac'
        :rtype: string

        '''
        return "hmac"

    @classmethod
    def getClassPrefix(cls):
        return "oath"

    @classmethod
    @log_with(log)
    def getClassInfo(cls, key=None, ret='all'):
        '''
        getClassInfo - returns a subtree of the token definition

        :param key: subsection identifier
        :type key: string

        :param ret: default return value, if nothing is found
        :type ret: user defined

        :return: subsection if key exists or user defined
        :rtype: s.o.

        '''
        desc_self1 = _('Specify the hashlib to be used. '
                       'Can be sha1 (1) or sha2-256 (2).')
        desc_self2 = _('Specify the otplen to be used. Can be 6 or 8 digits.')
        res = {'type': 'hmac',
               'title': 'HOTP Event Token',
               'description': ('event based otp token using the HOTP '
                               'algorithm'),
               'init': {'page': {'html': 'hmactoken.mako',
                                 'scope': 'enroll', },
                        'title': {'html': 'hmactoken.mako',
                                  'scope': 'enroll.title', },
                        },
               'config': {'page': {'html': 'hmactoken.mako',
                                   'scope': 'config', },
                          'title': {'html': 'hmactoken.mako',
                                    'scope': 'config.title', },
                          },
               'selfservice': {'enroll': {'page': {'html': 'hmactoken.mako',
                                                   'scope': 'selfservice.'
                                                   'enroll', },
                                          'title': {'html': 'hmactoken.mako',
                                                    'scope': 'selfservice.'
                                                    'title.enroll', },
                                          },
                               },
               'policy': {'selfservice': {'hmac_hashlib': {'type': 'int',
                                                           'value': [1, 2],
                                                           'desc': desc_self1
                                                           },
                                          'hmac_otplen': {'type': 'int',
                                                          'value': [6, 8],
                                                          'desc': desc_self2
                                                          },
                                          }
                          }
               }

        if key is not None and key in res:
            ret = res.get(key)
        else:
            if ret == 'all':
                ret = res
        return ret

    @log_with(log)
    def __init__(self, a_token):
        '''
        constructor - create a token object

        :param aToken: instance of the orm db object
        :type aToken:  orm object

        '''

        TokenClass.__init__(self, a_token)
        self.setType(u"HMAC")
        self.hKeyRequired = True

        # we support various hashlib methods, but only on create
        # which is effectively set in the update

        self.hashlibStr = u"sha1"
        try:
            self.hashlibStr = getFromConfig("hotp.hashlib", u'sha1')
        except Exception as ex:
            log.error('Failed to get the hotp.hashlib (%r)' % (ex))
            raise Exception(ex)

        return

    @log_with(log)
    def getInitDetail(self, params, user=None):
        '''
        to complete the token normalisation, the response of the initialiastion
        should be build by the token specific method, the getInitDetails
        '''
        response_detail = TokenClass.getInitDetail(self, params, user)
        otpkey = self.getInfo().get('otpkey')
        if otpkey:
            tok_type = self.type.lower()
            if user is not None:
                try:
                    goo_url = cr_google(user.login,
                                        user.realm,
                                        otpkey,
                                        tok_type.lower(),
                                        serial=self.getSerial())
                    response_detail["googleurl"] = {"description":
                                                    _("URL for google "
                                                      "Authenticator"),
                                                    "value": goo_url,
                                                    "img": create_img(goo_url,
                                                                      width=250)
                                                    }

                    oath_url = cr_oath(user.login,
                                       user.realm,
                                       otpkey, tok_type,
                                       serial=self.getSerial())
                    response_detail["oathurl"] = {"description": _("URL for"
                                                                   " OATH "
                                                                   "token"),
                                                  "value": oath_url,
                                                  "img": create_img(oath_url,
                                                                    width=250)
                                                  }
                except Exception as ex:
                    log.error("%r" % (traceback.format_exc()))
                    log.error('failed to set oath or google url: %r' % ex)
                    
        return response_detail

    @log_with(log)
    def update(self, param, reset_failcount=True):
        '''
        update - process the initialization parameters

        :param param: dict of initialization parameters
        :type param: dict

        :return: nothing
        '''
        # Remark: the otpKey is handled in the parent class

        val = getParam(param, "hashlib", optional)
        if val is not None:
            self.hashlibStr = val
        else:
            self.hashlibStr = 'sha1'

        # check if the key_size id provided
        # if not, we could derive it from the hashlib
        key_size = getParam(param, 'key_size', optional)
        if key_size is None:
            param['key_size'] = keylen.get(self.hashlibStr)

        param['hashlib'] = self.hashlibStr
        self.addToTokenInfo("hashlib", self.hashlibStr)

        TokenClass.update(self, param, reset_failcount)
        return
        
    # challenge interfaces starts here
    @log_with(log)
    def is_challenge_request(self, passw, user, options=None):
        '''
        check, if the request would start a challenge

        - default: if the passw contains only the pin, this request would
        trigger a challenge

        - in this place as well the policy for a token is checked

        :param passw: password, which might be pin or pin+otp
        :param options: dictionary of additional request parameters

        :return: returns true or false
        '''

        trigger_challenge = False
        pin_match = check_pin(self, passw, user=user, options=options)
        if pin_match is True:
            trigger_challenge = True

        return trigger_challenge

    @log_with(log)
    def checkResponse4Challenge(self, user, passw, options=None,
                                challenges=None):
        '''
        verify the response of a previous challenge

        :param user:     the requesting user
        :param passw:    the to be checked pass (pin+otp)
        :param options:  options an additional argument, which could be token
                          specific
        :param challenges: the list of challenges, where each challenge is
                            described as dict
        :return: tuple of (otpcounter and the list of matching challenges)

        '''
        otp_counter = -1
        transid = None
        matching = None
        matchin_challenges = []

        if 'transactionid' in options or 'state' in options:
            # fetch the transactionid
            transid = options.get('transactionid', None)
            if transid is None:
                transid = options.get('state', None)

        # check if the transactionid is in the list of challenges
        if transid is not None:
            for challenge in challenges:
                if challenge.getTransactionId() == transid:
                    matching = challenge
                    break
            if matching is not None:
                otp_counter = check_otp(self, passw, options=options)
                if otp_counter >= 0:
                    matchin_challenges.append(matching)

        return (otp_counter, matchin_challenges)

    @log_with(log)
    def createChallenge(self, state, options=None):
        '''
        create a challenge, which is submitted to the user

        :param state: the state/transaction id
        :param options: the request context parameters / data
        :return: tuple of (bool, message and data)
                 message is submitted to the user
                 data is preserved in the challenge
                 attributes are additional attributes, which could be returned
        '''
        message = 'Please enter your otp value: '
        data = {'serial': self.token.getSerial(),
                'date': "%s" % datetime.now()
                }

        return (True, message, data, None)

    @log_with(log)
    def checkOtp(self, anOtpVal, counter, window, options=None):
        '''
        checkOtp - validate the token otp against a given otpvalue

        :param anOtpVal: the to be verified otpvalue
        :type anOtpVal:  string

        :param counter: the counter state, that should be verified
        :type counter: int

        :param window: the counter +window, which should be checked
        :type window: int

        :param options: the dict, which could contain token specific info
        :type options: dict

        :return: the counter state or -1
        :rtype: int

        '''
        try:
            otplen = int(self.getOtpLen())
        except ValueError as ex:
            log.error('failed to initialize otplen: ValueError %r %r' %
                      (ex, self.token.privacyIDEAOtpLen))
            raise Exception(ex)

        try:
            self.hashlibStr = self.getFromTokenInfo("hashlib", 'sha1')
        except Exception as ex:
            log.error('failed to initialize hashlibStr: %r' % (ex))
            raise Exception(ex)

        secretHOtp = self.token.getHOtpKey()
        # serialNum   = self.token.privacyIDEATokenSerialnumber
        # log.debug("serial: %s",serialNum)

        hmac2Otp = HmacOtp(secretHOtp,
                           counter,
                           otplen,
                           self.getHashlib(self.hashlibStr))
        res = hmac2Otp.checkOtp(anOtpVal, window)

        if -1 == res:
            res = self.autosync(hmac2Otp, anOtpVal)

        return res

    @log_with(log)
    def check_otp_exist(self, otp, window=10):
        '''
        checks if the given OTP value is/are values of this very token.
        This is used to autoassign and to determine the serial number of
        a token.

        :param otp: the to be verified otp value
        :type otp: string

        :param window: the lookahead window for the counter
        :type window: int

        :return: counter or -1 if otp does not exist
        :rtype:  int

        '''
        res = -1

        try:
            otplen = int(self.token.privacyIDEAOtpLen)
            counter = int(self.token.privacyIDEACount)
        except ValueError as ex:
            log.warning("a value error occurred while converting: otplen %r, "
                        "counter %r : ValueError: %r ret: %r "
                        % (self.token.privacyIDEAOtpLen,
                           self.token.privacyIDEACount, ex, res))
            return res

        self.hashlibStr = self.getFromTokenInfo("hashlib", "sha1")

        secretHOtp = self.token.getHOtpKey()
        hmac2Otp = HmacOtp(secretHOtp, counter, otplen,
                           self.getHashlib(self.hashlibStr))
        res = hmac2Otp.checkOtp(otp, window)

        if res >= 0:
            # As usually the counter is increased in lib.token.checkUserPass,
            # we need to do this manually here:
            self.incOtpCounter(res)
        if res == -1:
            msg = "otp counter %r was not found" % otp
        else:
            msg = "otp counter %r was found" % otp
        log.debug("end. %r: res %r" % (msg, res))
        return res

    @log_with(log)
    def autosync(self, hmac2Otp, anOtpVal):
        '''
        auto - sync the token based on two otp values
        - internal method to realize the autosync within the
        checkOtp method

        :param hmac2Otp: the hmac object (with reference to the token secret)
        :type hmac2Otp: hmac object

        :param anOtpVal: the actual otp value
        :type anOtpVal: string

        :return: counter or -1 if otp does not exist
        :rtype:  int

        '''
        res = -1
        autosync = False

        # get autosync from config or use False as default
        async = getFromConfig("AutoResync", False)
        # The SQLite database returns AutoResync as a boolean and not as a
        # string. So the boolean has no .lower()
        if isinstance(async, bool):
            autosync = async
        else:
            if async.lower() == "true":
                autosync = True
            elif async.lower() == "false":
                autosync = False
            else:
                autosync = False

        # if autosync is enabled
        if autosync is False:
            log.debug("end. autosync is not enabled : res %r" % (res))
            return res

        info = self.getTokenInfo()
        syncWindow = self.getSyncWindow()

        # check if the otpval is valid in the sync scope
        res = hmac2Otp.checkOtp(anOtpVal, syncWindow)

        if res != -1:
            # if former is defined
            if ("otp1c" in info):
                # check if this is consecutive
                otp1c = info.get("otp1c")
                otp2c = res

                if (otp1c + 1) != otp2c:
                    res = -1

                if "dueDate" in info:
                    dueDate = info.get("dueDate")
                    now = int(time.time())
                    if dueDate <= now:
                        res = -1
                else:
                    res = -1

                # now clean the resync data
                del info["dueDate"]
                del info["otp1c"]
                self.setTokenInfo(info)

            else:
                info["otp1c"] = res
                info["dueDate"] = int(time.time()) + self.getSyncTimeOut()
                self.setTokenInfo(info)

                res = -1

        if res == -1:
            msg = "call was not successful"
        else:
            msg = "call was successful"
        log.debug("end. %r: res %r" % (msg, res))

        return res

    @log_with(log)
    def resync(self, otp1, otp2, options=None):
        '''
        resync the token based on two otp values
        - external method to do the resync of the token

        :param otp1: the first otp value
        :type otp1: string

        :param otp2: the second otp value
        :type otp2: string

        :param options: optional token specific parameters
        :type options:  dict or None

        :return: counter or -1 if otp does not exist
        :rtype:  int

        '''
        ret = False

        try:
            otplen = int(self.token.privacyIDEAOtpLen)
        except ValueError as ex:
            log.debug("otplen ValueError: %r ret: %r " % (ex, ret))
            raise Exception(ex)

        self.hashlibStr = self.getFromTokenInfo("hashlib", 'sha1')

        secretHOtp = self.token.getHOtpKey()
        counter = self.token.getOtpCounter()
        syncWindow = self.token.getSyncWindow()
        # log.debug("serial: %s",serialNum)
        hmac2Otp = HmacOtp(secretHOtp, counter, otplen,
                           self.getHashlib(self.hashlibStr))
        counter = hmac2Otp.checkOtp(otp1, syncWindow)

        if counter == -1:
            log.debug("exit. First counter (-1) not found  ret: %r" % (ret))
            return ret

        nextOtp = hmac2Otp.generate(counter + 1)

        if nextOtp != otp2:
            log.debug("exit. Failed to verify second otp: nextOtp: "
                      "%r != otp2: %r ret: %r" % (nextOtp, otp2, ret))
            return ret

        ret = True
        self.incOtpCounter(counter + 1, True)

        log.debug("end. resync was successful: ret: %r" % (ret))
        return ret

    def getSyncTimeOut(self):
        '''
        get the token sync timeout value

        :return: timeout value in seconds
        :rtype:  int
        '''
        try:
            timeOut = int(getFromConfig("AutoResyncTimeout", 5 * 60))
        except Exception as ex:
            log.warning("AutoResyncTimeout: value error %r - reset to 5*60"
                        % (ex))
            timeOut = 5 * 60

        return timeOut

    @log_with(log)
    def getOtp(self, curTime=None):
        '''
        get the next OTP value

        :return: next otp value
        :rtype: string
        '''
        try:
            otplen = int(self.token.privacyIDEAOtpLen)
        except ValueError as ex:
            log.error("Could not convert otplen - value error %r " % (ex))
            raise Exception(ex)

        self.hashlibStr = self.getFromTokenInfo("hashlib", 'sha1')
        secretHOtp = self.token.getHOtpKey()

        hmac2Otp = HmacOtp(secretHOtp,
                           self.getOtpCount(),
                           otplen,
                           self.getHashlib(self.hashlibStr))
        otpval = hmac2Otp.generate(inc_counter=False)

        pin = self.token.getPin()
        combined = "%s%s" % (otpval, pin)

        if getFromConfig("PrependPin") == "True":
            combined = "%s%s" % (pin, otpval)

        return (1, pin, otpval, combined)

    @log_with(log)
    def get_multi_otp(self, count=0, epoch_start=0, epoch_end=0,
                      curTime=None, timestamp=None):
        '''
        return a dictionary of multiple future OTP values of the
        HOTP/HMAC token

        :param count: how many otp values should be returned
        :type count: int

        :return: tuple of status: boolean, error: text and the OTP dictionary

        '''
        otp_dict = {"type": "HMAC", "otp": {}}
        ret = False
        error = "No count specified"
        try:
            otplen = int(self.token.privacyIDEAOtpLen)
        except ValueError as ex:
            log.error("Could not convert otplen - value error %r " % (ex))
            raise Exception(ex)

        secretHOtp = self.token.getHOtpKey()
        hmac2Otp = HmacOtp(secretHOtp, self.getOtpCount(), otplen,
                           self.getHashlib(self.hashlibStr))
        log.debug("retrieving %i OTP values for token %s" % (count, hmac2Otp))

        if count > 0:
            for i in range(count):
                otpval = hmac2Otp.generate(self.getOtpCount() + i,
                                           inc_counter=False)
                otp_dict["otp"][i] = otpval
            ret = True

        return (ret, error, otp_dict)



# -*- coding:utf-8 -*-
from mako import runtime, filters, cache
UNDEFINED = runtime.UNDEFINED
__M_dict_builtin = dict
__M_locals_builtin = locals
_magic_number = 9
_modified_time = 1422216484.916012
_enable_loop = True
_template_filename = '/usr/lib/python2.7/dist-packages/privacyidea/lib/tokens/hmactoken.mako'
_template_uri = '/hmactoken.mako'
_source_encoding = 'utf-8'
from webhelpers.html import escape
_exports = []


def render_body(context,**pageargs):
    __M_caller = context.caller_stack._push_frame()
    try:
        __M_locals = __M_dict_builtin(pageargs=pageargs)
        c = context.get('c', UNDEFINED)
        _ = context.get('_', UNDEFINED)
        __M_writer = context.writer()
        # SOURCE LINE 2
        __M_writer(u'\n\n')
        # SOURCE LINE 4
        if c.scope == 'config.title' :
            # SOURCE LINE 5
            __M_writer(u' ')
            __M_writer(escape(_("HMAC Token Settings")))
            __M_writer(u'\n')
        # SOURCE LINE 7
        __M_writer(u'\n\n')
        # SOURCE LINE 9
        if c.scope == 'config' :
            pass
        # SOURCE LINE 11
        __M_writer(u'\n\n')
        # SOURCE LINE 13
        if c.scope == 'enroll.title' :
            # SOURCE LINE 14
            __M_writer(escape(_("HMAC eventbased")))
            __M_writer(u'\n')
        # SOURCE LINE 16
        __M_writer(u'\n')
        # SOURCE LINE 17
        if c.scope == 'enroll' :
            # SOURCE LINE 18
            __M_writer(u"<script>\n\n/*\n * 'typ'_get_enroll_params()\n *\n * this method is called, when the token  is submitted\n * - it will return a hash of parameters for admin/init call\n *\n */\n\nfunction hmac_get_enroll_params(){\n    var url = {};\n    url['type'] = 'hmac';\n   \turl['description'] = $('#enroll_hmac_desc').val();\n\n    // If we have got to generate the hmac key, we do it here:\n    if  ( $('#hmac_key_cb').attr('checked') ) {\n    \turl['genkey'] = 1;\n\n    } else {\n        // OTP Key\n        url['otpkey'] = $('#hmac_key').val();\n    }\n\n    jQuery.extend(url, add_user_data());\n\n    url['hashlib']\t= $('#hmac_algorithm').val();\n\turl['otplen']\t= $('#hmac_otplen').val();\n\n\n    return url;\n}\n</script>\n\n<p><span id='hmac_key_intro'>\n\t")
            # SOURCE LINE 53
            __M_writer(escape(_("Please enter or copy the HMAC key.")))
            __M_writer(u'</span></p>\n<table><tr>\n\t<td><label for="hmac_key" id=\'hmac_key_label\'>')
            # SOURCE LINE 55
            __M_writer(escape(_("HMAC key")))
            __M_writer(u'</label></td>\n\t<td><input type="text" name="hmac_key" id="hmac_key" value="" class="text ui-widget-content ui-corner-all" /></td>\n</tr><tr>\n\t<td> </td><td><input type=\'checkbox\' id=\'hmac_key_cb\' onclick="cb_changed(\'hmac_key_cb\',[\'hmac_key\',\'hmac_key_label\',\'hmac_key_intro\']);">\n\t<label for=hmac_key_cb>')
            # SOURCE LINE 59
            __M_writer(escape(_("Generate HMAC key.")))
            __M_writer(u'</label></td>\n</tr><tr>\n\t<td><label for="hmac_otplen">')
            # SOURCE LINE 61
            __M_writer(escape(_("OTP Length")))
            __M_writer(u'</label></td>\n\t<td><select name="pintype" id="hmac_otplen">\n\t\t\t<option  value="4">4</option>\n\t\t\t<option  selected value="6">6</option>\n\t\t\t<option  value="8">8</option>\n\t\t\t<option  value="10">10</option>\n\t\t\t<option  value="12">12</option>\n\t</select></td>\n\n</tr><tr>\n\t<td><label for="hmac_algorithm">')
            # SOURCE LINE 71
            __M_writer(escape(_("Hash algorithm")))
            __M_writer(u'</label></td>\n\t<td><select name="algorithm" id=\'hmac_algorithm\' >\n\t        <option selected value="sha1">sha1</option>\n\t        <option value="sha256">sha256</option>\n\t        <option value="sha512">sha512</option>\n    </select></td>\n</tr>\n<tr>\n    <td><label for="enroll_hmac_desc" id=\'enroll_hmac_desc_label\'>')
            # SOURCE LINE 79
            __M_writer(escape(_("Description")))
            __M_writer(u'</label></td>\n    <td><input type="text" name="enroll_hmac_desc" id="enroll_hmac_desc" value="webGUI_generated" class="text" /></td>\n</tr>\n\n</table>\n\n')
        # SOURCE LINE 86
        __M_writer(u'\n\n\n\n')
        # SOURCE LINE 90
        if c.scope == 'selfservice.title.enroll':
            # SOURCE LINE 91
            __M_writer(escape(_("Enroll your HOTP Token")))
            __M_writer(u'\n')
        # SOURCE LINE 93
        __M_writer(u'\n\n')
        # SOURCE LINE 95
        if c.scope == 'selfservice.enroll':
            # SOURCE LINE 96
            __M_writer(u'<script>\n\tjQuery.extend(jQuery.validator.messages, {\n\t\trequired: "')
            # SOURCE LINE 98
            __M_writer(escape(_('required input field')))
            __M_writer(u'",\n\t\tminlength: "')
            # SOURCE LINE 99
            __M_writer(escape(_('minimum length must be greater than {0}')))
            __M_writer(u'",\n\t\tmaxlength: "')
            # SOURCE LINE 100
            __M_writer(escape(_('maximum length must be lower than {0}')))
            __M_writer(u'",\n\n\t});\n\njQuery.validator.addMethod("hmac_secret", function(value, element, param){\n\tvar res1 = value.match(/^[a-fA-F0-9]+$/i);\n\tvar res2 = !value;\n    return  res1 || res2 ;\n}, \'')
            # SOURCE LINE 108
            __M_writer(escape(_("Please enter a valid init secret. It may only contain numbers and the letters A-F.")))
            __M_writer(u'\'  );\n\n$(\'#form_enroll_hmac\').validate({\n\tdebug: true,\n    rules: {\n        hmac_secret: {\n            minlength: 40,\n            maxlength: 64,\n            number: false,\n            hmac_secret: true,\n\t\t\trequired: function() {\n            \tvar res = $(\'#hmac_key_cb2\').attr(\'checked\') === \'undefined\';\n            return res;\n        }\n        }\n    }\n});\n\nfunction self_hmac_get_param()\n{\n\tvar urlparam = {};\n\tvar typ = \'hmac\';\n\n    if  ( $(\'#hmac_key_cb2\').attr(\'checked\') ) {\n    \turlparam[\'genkey\'] = 1;\n    } else {\n        // OTP Key\n        urlparam[\'otpkey\'] = $(\'#hmac_secret\').val();\n    }\n\n\turlparam[\'type\'] \t= typ;\n\turlparam[\'hashlib\'] = $(\'#hmac_hashlib\').val();\n\turlparam[\'otplen\'] \t= 6;\n\turlparam[\'description\'] = $("#hmac_self_desc").val();\n\n\treturn urlparam;\n}\n\nfunction self_hmac_clear()\n{\n\t$(\'#hmac_secret\').val(\'\');\n\n}\nfunction self_hmac_submit(){\n\n\tvar ret = false;\n\tvar params =  self_hmac_get_param();\n\n\tif  ( $(\'#hmac_key_cb2\').attr(\'checked\') === undefined && $(\'#form_enroll_hmac\').valid() === false) {\n\t\talert(\'')
            # SOURCE LINE 157
            __M_writer(escape(_("Form data not valid.")))
            __M_writer(u'\');\n\t} else {\n\t\tenroll_token( params );\n\t\t$("#hmac_key_cb2").prop("checked", false);\n\t\tcb_changed(\'hmac_key_cb2\',[\'hmac_secret\',\'hmac_key_label2\']);\n\t\tret = true;\n\t}\n\treturn ret;\n\n}\n</script>\n<h1>')
            # SOURCE LINE 168
            __M_writer(escape(_("Enroll your HOTP Token")))
            __M_writer(u'</h1>\n<div id=\'enroll_hmac_form\'>\n\t<form class="cmxform" id=\'form_enroll_hmac\'>\n\t<fieldset>\n\t\t<table><tr>\n\t\t\t<td><label for=\'hmac_key_cb\' id=\'hmac_key_label2\'>')
            # SOURCE LINE 173
            __M_writer(escape(_("Generate HMAC key")+':'))
            __M_writer(u'</label></td>\n\t\t\t<td><input type=\'checkbox\' name=\'hmac_key_cb2\' id=\'hmac_key_cb2\' onclick="cb_changed(\'hmac_key_cb2\',[\'hmac_secret\',\'hmac_key_label2\']);"></td>\n\t\t</tr><tr>\n\t\t\t<td><label id=\'hmac_key_label2\' for=\'hmac_secret\'>')
            # SOURCE LINE 176
            __M_writer(escape(_("Seed for HOTP token")))
            __M_writer(u'</label></td>\n\t\t\t<td><input id=\'hmac_secret\' name=\'hmac_secret\' class="required ui-widget-content ui-corner-all" min="40" maxlength=\'64\'/></td>\n\t\t</tr>\n')
            # SOURCE LINE 179
            if c.hmac_hashlib == 1:
                # SOURCE LINE 180
                __M_writer(u"\t\t\t<input type=hidden id='hmac_hashlib' name='hmac_hashlib' value='sha1'>\n")
            # SOURCE LINE 182
            if c.hmac_hashlib == 2:
                # SOURCE LINE 183
                __M_writer(u"\t\t\t<input type=hidden id='hmac_hashlib' name='hmac_hashlib' value='sha256'>\n")
            # SOURCE LINE 185
            if c.hmac_hashlib == -1:
                # SOURCE LINE 186
                __M_writer(u"\t\t<tr>\n\t\t<td><label for='hmac_hashlib'>")
                # SOURCE LINE 187
                __M_writer(escape(_("hashlib")))
                __M_writer(u"</label></td>\n\t\t<td><select id='hmac_hashlib' name='hmac_hashlib'>\n\t\t\t<option value='sha1' selected>sha1</option>\n\t\t\t<option value='sha256'>sha256</option>\n\t\t\t</select></td>\n\t\t</tr>\n")
            # SOURCE LINE 194
            __M_writer(u'\t\t<tr>\n\t\t    <td><label for="hmac_self_desc" id=\'hmac_self_desc_label\'>')
            # SOURCE LINE 195
            __M_writer(escape(_("Description")))
            __M_writer(u'</label></td>\n\t\t    <td><input type="text" name="hmac_self_desc" id="hmac_self_desc" value="self enrolled" class="text" /></td>\n\t\t</tr>\n\n        </table>\n\t    <button class=\'action-button\' id=\'button_enroll_hmac\'\n\t    \t    onclick="self_hmac_submit();">')
            # SOURCE LINE 201
            __M_writer(escape(_("enroll hmac token")))
            __M_writer(u'</button>\n\n    </fieldset>\n    </form>\n</div>\n\n')
        return ''
    finally:
        context.caller_stack._pop_frame()


